/********************************************************
*
* File:  intertechno.c
* Titel: include fr alle Intertechno Untersttzung
* Version: 0.1
* Stand: 30.5.2009
*
*
* Autor: Stefan Andres
* email: stefan@myandres.de
* Web: www.myandres.de
*
* Steuert einen 433Mhz SAW Sender mit den Code fr
*  INTERTECHNO
* Funkgerte an.
*
* Signal wird am Prozessor auf PORTC/PIN0 ausgegeben.
*
*
*********************************************************/



#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "tools.h"
#include "main.h" //alle Zentralen Definitionen
#include "uart.h"

#ifndef F_CPU
// F_CPU nicht definiert
#error "F_CPU nicht definiert"
#endif

#include <util/delay.h>


void intertechno_init(void) {

DDRC |= ( 1 << PC0 );    // PIN PC0 auf Ausgang
PORTC &= ~( 1 << PC0 );  // PIN PC0 auf 0

}


void intertechno_send(uint16_t code) {


int i,k;
uint16_t codesave;
static char locs1[32];

codesave=code;
itoa(code,locs1,2);

for (k=0;k<6;k++) { //daten 6 mal senden
  code=codesave;
  for (i=1;i<13;i++) {
   // immer eine 0 raus
   PORTC |= ( 1 << PC0 ); // PIN PC0 auf 1
   _delay_us(370);
   PORTC &= ~( 1 << PC0 ); // PIN PC0 auf 0
   _delay_ms(1);
   if ((code & 1) == 0) {   // 0 senden
     PORTC |= ( 1 << PC0 );  // PIN PC0 auf 1
     _delay_us(370);
     PORTC &= ~( 1 << PC0 ); // PIN PC0 auf 0
     _delay_ms(1); }
   else {                    // 1 senden
     PORTC |= ( 1 << PC0 );  // PIN PC0 auf 1
     _delay_ms(1);
     PORTC &= ~( 1 << PC0 ); // PIN PC0 auf 0
     _delay_us(370); }

   code >>= 1;
  } //for i
  //syncbit senden
  PORTC |= ( 1 << PC0 ); // PIN PC0 auf 1
  _delay_us(370);
  PORTC &= ~( 1 << PC0 ); // PIN PC0 auf 0
  _delay_ms(1);
  _delay_ms(12); //warte 32*370ys
} //for k
   uart_write("code send: ");
   uart_writeln(locs1);

}


int intertechno(char *sin) {

// Ermitteln der Parameter und senden des Devices

uint8_t familiencode;
uint8_t einzelcode;
uint8_t on_off;
uint16_t code;
static char locs[32];


        if (next_param(sin, locs, TRUE) == 0) {
          uart_writeln("ERROR: Familiencode fehlt");
      return -1;
    } //if;

    familiencode = *locs-65;

    if (!(familiencode<=16)) { //falser Familiencode
      uart_write("ERROR: Falscher Familiencode:");
      uart_writeln(locs);
      return -1;
    } //if


    if (next_param(sin, locs, FALSE) == 0) {
          uart_writeln("ERROR: Einzelcode fehlt");
      return -1;
    } //if;

    einzelcode=atoi(locs)-1;

    if (!(einzelcode<=16)) { //falser Einzelcode
      uart_write("ERROR: Falscher Einzelcode:");
      uart_writeln(locs);
      return -1;
    } //if

    if (next_param(sin, locs, TRUE) == 0) {
          uart_writeln("ERROR: on/off fehlt");
      return -1;
    } //if;


    on_off=0;
    if (strncmp(locs,"ON",2) == 0 ) on_off=0b00001110 ;
    if (strncmp(locs,"OFF",3) == 0 ) on_off=0b00000110 ;

    if (on_off == 0)  { //kein on off
      uart_write("ERROR: Falsches on/off:");
      uart_writeln(locs);
      return -1;
    } //if

   code = familiencode;
   code |= (einzelcode << 4);
   code |= (on_off<<8);

   uart_write("Intertechno: ");

   intertechno_send(code);


   return 0;
} //intertechno

