/********************************************************
*
* File:  uart.c
* Titel: Include for handling of UART
* Version: 0.1
* Stand: 26.12.2009
*
*
* Autor: Stefan Andres
* email: stefan@myandres.de
* Web: www.myandres.de
*
*
* Realisiert:
*
* void uart_init(void) :init uart
* void uart_putc(unsigned char ch) :send one char to UART
* void uart_newline (void) :send \r \n to UART
* void uart_write (char *s) :send string to UART
* void uart_writeln (char *s) :send string with \r \n to UART
*
* 
*********************************************************/

#include "main.h"
#include <avr/io.h>
#include <stdlib.h>


#ifndef F_CPU
// F_CPU nicht definiert
#error "F_CPU nicht definiert"
#endif

#ifndef UART_BAUD
// Baudrate fr UART wurde nicht definiert
#warning "UART_BAUD nicht definiert: Setze UART_BAUD=9600UL"
#define UART_BAUD 9600UL          // Baudrate
#endif

// Berechnungen
#define UBRR_VAL ((F_CPU+UART_BAUD*8)/(UART_BAUD*16)-1)   // clever runden
#define BAUD_REAL (F_CPU/(16*(UBRR_VAL+1)))     // Reale Baudrate
#define BAUD_ERROR ((BAUD_REAL*1000)/UART_BAUD) // Fehler in Promille, 1000 = kein Fehler.

#if ((BAUD_ERROR<990) || (BAUD_ERROR>1010))
  #error Systematischer Fehler der Baudrate grsser 1% und damit zu hoch!
#endif


void uart_init(void) {

// Init des UART.
   UCSRB |= (1<<RXCIE)|(1<<TXEN)|(1<<RXEN); // UART INT TX RX einschalten
   UCSRC |= (1<<URSEL)|(3<<UCSZ0);    // Asynchron 8N1


// nur fr grer atmega8
   UBRRH = UBRR_VAL >> 8;             // Baudrate einstellen
   UBRRL = UBRR_VAL & 0xFF;

}

void uart_putc(unsigned char ch) {

while (!(UCSRA & (1<<UDRE))) {}  //warten bis senden OK

UDR = ch ;  //sende char

}

void uart_newline (void) {
    uart_putc('\r'); //Return
    uart_putc('\n'); //Newline
}

void uart_write (char *s)
{
        while (*s)
    {   /* so lange *s != '\0' also ungleich dem "String-Endezeichen" */
        uart_putc(*s);
        s++;
    }

}

void uart_writeln (char *s)
{
        uart_write(s);
        uart_newline();

}


//END uart.c
